@* The Newton direction. If you are lucky enough that the Hessian computation is not a big deal
and you can afford solving a linear system each time the search direction is needed, then the
Newton direction is a no-brainer. The only thing that needs to be defined is what type of storage 
you want to use for the Hessian. For large, sparse problems, you probably want a sparse storage.
@c
/* Empty, waiting for |export|. */
@ @(searchdir/newton.h@>=
#ifndef __MATH_SEARCHDIR_NEWTON__
#define __MATH_SEARCHDIR_NEWTON__ 1.0
#include <math/searchdirbase.h>
#include <math/algebra.h>
#include <math/lu.h>
#include <math/cholesky.h>
namespace math
{
    @[namespace searchdir@]
    {
	template<class T,template<@[class@]> class @[storage@]>@;
	class @[newton : public base<T>@]
	{
	    @<Newton search direction internal variables@>@;
	public:@/
	    @<Newton search direction methods@>@;
	};
    }
}
#endif
@ We assume that you'll need to compute the search direction many times, so we reserve space for
the Hessian.
@<Newton search direction internal variables@>=
matrix<T,symmetric,storage> hess;
@ The search direction is now~$-\inv(\nabla^2f)\nabla f$. We first compute the gradient and store
it in~$\\{aux}$, following by a Hessian computation and Cholesky linear system solver. In case
the Hessian is singular or close, we return the gradient direction (which is still a descent
direction).
@<Newton search direction methods@>=@!@:searchdir::newton}{dir@>
matrix<T,unstructured,dense>&
dir(functional::base<T>* f,const matrix<T,unstructured,dense>& x,
	matrix<T,unstructured,dense>* dest)
{
    try
    {
	try
	{
	    f->grad(x,*dest);
	    f->hess(x,hess);
	    cholesky::solve(&hess,dest);
	}
	catch(error::nonpositivedef e)
	{
	    f->grad(x,*dest);
	    f->hess(x,hess);
	    lu::solve(&hess,dest);
	}
    }
    catch(error::singular e)@+ {@+ f->grad(x,*dest);@+ }
    return (*dest)*=-1;
}
